@@ -31,6 +31,8 @@ import warnings
 import json
 import hashlib
 import re
+import copy
+import functools
 
 from luigi import six
 
@@ -51,12 +53,10 @@ def namespace(namespace=None):
     """
     Call to set namespace of tasks declared after the call.
 
-    If called without arguments or with ``None`` as the namespace, the namespace
-    is reset, which is recommended to do at the end of any file where the
-    namespace is set to avoid unintentionally setting namespace on tasks outside
-    of the scope of the current file.
+    It is best practice to call this function without arguments at the end of any file it has been
+    used in. That is to ensure that subsequent tasks have the default namespace again.
 
-    The namespace of a Task can also be changed by specifying the property
+    The namespace of a :py:class:`Task` can also be changed by specifying the property
     ``task_namespace``. This solution has the advantage that the namespace
     doesn't have to be restored.
 
@@ -64,8 +64,10 @@ def namespace(namespace=None):
 
         class Task2(luigi.Task):
             task_namespace = 'namespace2'
+
+    There's no equivalent way to set the ``task_family``.
     """
-    Register._default_namespace = namespace
+    Register._default_namespace = namespace or ''
 
 
 def task_id_str(task_family, params):
@@ -123,10 +125,6 @@ class Task(object):
     non-declared properties, which are created by the :py:class:`Register`
     metaclass:
 
-    ``Task.task_namespace``
-      optional string which is prepended to the task name for the sake of
-      scheduling. If it isn't overridden in a Task, whatever was last declared
-      using `luigi.namespace` will be used.
     """
 
     _event_callbacks = {}
@@ -241,13 +239,58 @@ class Task(object):
         # TODO(erikbern): we should think about a language-agnostic mechanism
         return self.__class__.__module__
 
+    _visible_in_registry = True  # TODO: Consider using in luigi.util as well
+
+    __not_user_specified = '__not_user_specified'
+
+    task_namespace = __not_user_specified
+    """
+    This value can be overriden to set the namespace that will be used.
+    (See :ref:`Task.namespaces_famlies_and_ids`)
+    If it's not specified and you try to read this value anyway, it will return
+    garbage. Please use :py:meth:`get_task_namespace` to read the namespace.
+
+    Note that setting this value with ``@property`` will not work, because this
+    is a class level value.
+    """
+
+    @classmethod
+    def get_task_namespace(cls):
+        """
+        The task family for the given class.
+
+        Note: You normally don't want to override this.
+        """
+        if cls.task_namespace != cls.__not_user_specified:
+            return cls.task_namespace
+        return cls._namespace_at_class_time
+
     @property
     def task_family(self):
         """
-        Convenience method since a property on the metaclass isn't directly accessible through the class instances.
+        DEPRECATED since after 2.4.0. See :py:meth:`get_task_family` instead.
+        Hopefully there will be less meta magic in Luigi.
+
+        Convenience method since a property on the metaclass isn't directly
+        accessible through the class instances.
         """
         return self.__class__.task_family
 
+    @classmethod
+    def get_task_family(cls):
+        """
+        The task family for the given class.
+
+        If ``task_namespace`` is not set, then it's simply the name of the
+        class.  Otherwise, ``<task_namespace>.`` is prefixed to the class name.
+
+        Note: You normally don't want to override this.
+        """
+        if not cls.get_task_namespace():
+            return cls.__name__
+        else:
+            return "{}.{}".format(cls.get_task_namespace(), cls.__name__)
+
     @classmethod
     def get_params(cls):
         """
@@ -288,11 +331,11 @@ class Task(object):
 
         params_dict = dict(params)
 
-        task_name = cls.task_family
+        task_family = cls.get_task_family()
 
         # In case any exceptions are thrown, create a helpful description of how the Task was invoked
         # TODO: should we detect non-reprable arguments? These will lead to mysterious errors
-        exc_desc = '%s[args=%s, kwargs=%s]' % (task_name, args, kwargs)
+        exc_desc = '%s[args=%s, kwargs=%s]' % (task_family, args, kwargs)
 
         # Fill in the positional arguments
         positional_params = [(n, p) for n, p in params if p.positional]
@@ -313,9 +356,9 @@ class Task(object):
         # Then use the defaults for anything not filled in
         for param_name, param_obj in params:
             if param_name not in result:
-                if not param_obj.has_task_value(task_name, param_name):
+                if not param_obj.has_task_value(task_family, param_name):
                     raise parameter.MissingParameterException("%s: requires the '%s' parameter to be set" % (exc_desc, param_name))
-                result[param_name] = param_obj.task_value(task_name, param_name)
+                result[param_name] = param_obj.task_value(task_family, param_name)
 
         def list_to_tuple(x):
             """ Make tuples out of lists and sets to allow hashing """
@@ -338,7 +381,7 @@ class Task(object):
         self.param_args = tuple(value for key, value in param_values)
         self.param_kwargs = dict(param_values)
 
-        self.task_id = task_id_str(self.task_family, self.to_str_params(only_significant=True))
+        self.task_id = task_id_str(self.get_task_family(), self.to_str_params(only_significant=True))
         self.__hash = hash(self.task_id)
 
         self.set_tracking_url = None
@@ -422,7 +465,7 @@ class Task(object):
             if param_objs[param_name].significant:
                 repr_parts.append('%s=%s' % (param_name, param_objs[param_name].serialize(param_value)))
 
-        task_str = '{}({})'.format(self.task_family, ', '.join(repr_parts))
+        task_str = '{}({})'.format(self.get_task_family(), ', '.join(repr_parts))
 
         return task_str
 
@@ -618,16 +661,6 @@ class MixinNaiveBulkComplete(object):
         return generated_tuples
 
 
-def externalize(task):
-    """
-    Returns an externalized version of the Task.
-
-    See :py:class:`ExternalTask`.
-    """
-    task.run = None
-    return task
-
-
 class ExternalTask(Task):
     """
     Subclass for references to external dependencies.
@@ -639,6 +672,69 @@ class ExternalTask(Task):
     run = None
 
 
+def externalize(taskclass_or_taskobject):
+    """
+    Returns an externalized version of a Task. You may both pass an
+    instantiated task object or a task class. Some examples:
+
+    .. code-block:: python
+
+        class RequiringTask(luigi.Task):
+            def requires(self):
+                task_object = self.clone(MyTask)
+                return externalize(task_object)
+
+            ...
+
+    Here's mostly equivalent code, but ``externalize`` is applied to a task
+    class instead.
+
+    .. code-block:: python
+
+        @luigi.util.requires(externalize(MyTask))
+        class RequiringTask(luigi.Task):
+            pass
+            ...
+
+    Of course, it may also be used directly on classes and objects (for example
+    for reexporting or other usage).
+
+    .. code-block:: python
+
+        MyTask = externalize(MyTask)
+        my_task_2 = externalize(MyTask2(param='foo'))
+
+    If you however want a task class to be external from the beginning, you're
+    better off inheriting :py:class:`ExternalTask` rather than :py:class:`Task`.
+
+    This function tries to be side-effect free by creating a copy of the class
+    or the object passed in and then modify that object. In particular this
+    code shouldn't do anything.
+
+    .. code-block:: python
+
+        externalize(MyTask)  # BAD: This does nothing (as after luigi 2.4.0)
+    """
+    # Seems like with python < 3.3 copy.copy can't copy classes
+    # and objects with specified metaclass http://bugs.python.org/issue11480
+    compatible_copy = copy.copy if six.PY3 else copy.deepcopy
+    copied_value = compatible_copy(taskclass_or_taskobject)
+    if copied_value is taskclass_or_taskobject:
+        # Assume it's a class
+        clazz = taskclass_or_taskobject
+
+        @_task_wraps(clazz)
+        class _CopyOfClass(clazz):
+            # How to copy a class: http://stackoverflow.com/a/9541120/621449
+            _visible_in_registry = False
+        _CopyOfClass.run = None
+        return _CopyOfClass
+    else:
+        # We assume it's an object
+        copied_value.run = None
+        return copied_value
+
+
 class WrapperTask(Task):
     """
     Use for tasks that only wrap other tasks and that by definition are done if all their requirements exist.
@@ -725,3 +821,13 @@ def flatten_output(task):
         for dep in flatten(task.requires()):
             r += flatten_output(dep)
     return r
+
+
+def _task_wraps(task_class):
+    # In order to make the behavior of a wrapper class nicer, we set the name of the
+    # new class to the wrapped class, and copy over the docstring and module as well.
+    # This makes it possible to pickle the wrapped class etc.
+    # Btw, this is a slight abuse of functools.wraps. It's meant to be used only for
+    # functions, but it works for classes too, if you pass updated=[]
+    assigned = functools.WRAPPER_ASSIGNMENTS + ('_namespace_at_class_time',)
+    return functools.wraps(task_class, assigned=assigned, updated=[])
